//---------------------------------------------------------------------------
//pomocn knihovna
//obsahuje knihovnu td sloucch pro generovn VRML souboru
//napsal Jan Havrlnat jako soust diplomov prce
//--Automatizovan tvorba prostorovch model map--
//v roce 2002
//---------------------------------------------------------------------------
#ifndef VrmlformH
#define VrmlformH
#include <vector>
#include <math.h>

class Hbod3D;
class troj;
//---------------------------------------------------------------------------
//Bzov tda umoujc vstup jednotlivch uzl modelu do souboru
class vrml_node
{
  public:
  virtual void Print(ofstream &soubor) =0;

};
//Tda obsahujc vechny uzly v souboru
class vrml_file
{
  vrml_node** node;
  int num;
  public:
  vrml_file(){num=0;node=new vrml_node*[20];}
  vrml_file(int k){num=0;node=new vrml_node*[k];}
  //vrml_file(){node = new vrml_base[20];}
  void Add_node(vrml_node* n){node[num++] = n;}
  void Print (ofstream &soubor);
};
//---------------------------------------------------------------------------

//struktura pro definici barvy
struct vrml_color
{
  double r,g,b;
  vrml_color(double ra,double ga,double ba){r=ra;g=ga;b=ba;}
  vrml_color(){r=1;g=1;b=1;}
  friend ostream& operator << (ostream& o, vrml_color& s)
  {
    return o<<s.r<<" "<<s.g<<" "<<s.b;
  }

  vrml_color(int ba){ b =((16711680&ba)>>16)/255.0;g=((65280&ba)>>8)/255.0; r=(255&ba)/255.0;}

}  ;
//---------------------------------------------------------------------------
//sturktura pro definici vektoru
struct vrml_Vect3f
{
  float x,y,z;
  vrml_Vect3f(){}
  vrml_Vect3f(float Dx,float Dy,float Dz){x=Dx;y=Dy;z=Dz;}
  friend ostream& operator << (ostream& o, vrml_Vect3f& r)
  {
    return o<<r.x<<" "<<r.y<<" "<<r.z;
  }
};
//---------------------------------------------------------------------------
//sturktura pro definici vektoru
struct vrml_Vect2f
{
  float x,y;
  vrml_Vect2f(){}
  vrml_Vect2f(float Dx,float Dy){x=Dx;y=Dy;}
  friend ostream& operator << (ostream& o, vrml_Vect2f& r)
  {
    return o<<r.x<<" "<<r.y;
  }


};
//---------------------------------------------------------------------------
//sturktura pro definici rotace
struct vrml_Rotation
{
  float x,y,z,r;
  vrml_Rotation(){x=0;y=0;y=1;r=0;}
  vrml_Rotation(float dx,float dy,float dz,float dr){x=dx;y=dy;z=dz;r=dr;}
  friend ostream& operator << (ostream& o, vrml_Rotation& s)
  {
    return o<<s.x<<" "<<s.y<<" "<<s.z<<" "<<s.r;
  }

} ;
//---------------------------------------------------------------------------
//sturktura pro definici promnn BOOL
struct vrml_Bool
{
  bool vBool;
  friend ostream& operator << (ostream& o, vrml_Bool& r)
  {
    if(r.vBool) return o<<"TRUE";
    else return o<<"FALSE";
  }
  vrml_Bool(){;}
  vrml_Bool(bool b){vBool=b;}
  operator bool() {return vBool;}

};
//---------------------------------------------------------------------------
//definice jednotlivch uzl VRML jako tdy
class VRML_NAVIGATIONINFO : public vrml_node
{
  public:
  vrml_Vect3f avatarSize;
  vrml_Bool headlight;
  float speed;
  String type;
  float visibilityLimit;

  void Print(ofstream &soubor); //metoda pro uloen uzlu do souboru
  VRML_NAVIGATIONINFO ()
  {
    avatarSize=vrml_Vect3f(0.2,10,1);
    headlight=false;
    speed=100;
    type="\"WALK\",\"FLY\"";
    visibilityLimit=0;
  }

}  ;
//---------------------------------------------------------------------------

class  VRML_BACKGROUND : public vrml_node
{
  public:
  vrml_color* skyColor;
  int pocet;
  float* skyAngle;
  vrml_color* groundColor;
  int pocet1;
  float* groundAngle;
  void Print(ofstream &soubor);
  VRML_BACKGROUND ()
  {
    pocet=3;
    skyColor=new  vrml_color[pocet];
    skyAngle=new float[pocet-1];
    pocet1=3;
    groundColor=new vrml_color[pocet1];
    groundAngle=new float[pocet1-1];
  }
  VRML_BACKGROUND (vrml_color s1,vrml_color s2,vrml_color s3,float cs1,float cs2
                   ,vrml_color b1,vrml_color b2,vrml_color b3,float cs3,float cs4)
  {
    pocet=3;
    skyColor=new  vrml_color[pocet];
    skyColor[0]=s1;
    skyColor[1]=s2;
    skyColor[2]=s3;
    skyAngle=new float[pocet-1];
    skyAngle[0]=cs1;
    skyAngle[1]=cs2;
    pocet1=3;
    groundColor=new vrml_color[pocet1];
    groundColor[0]=b1;
    groundColor[1]=b2;
    groundColor[2]=b3;
    groundAngle=new float[pocet1-1];
    groundAngle[0]=cs3;
    groundAngle[1]=cs4;

  }
  ~VRML_BACKGROUND()
  {
    delete[] skyColor;   delete[] skyAngle;
    delete[] groundColor;
    delete[]  groundAngle;
  }

};
//---------------------------------------------------------------------------
class VRML_POINTLIGHT : public vrml_node
{
  public:
  vrml_Vect3f location;
  float radius;
  vrml_color color;
  float ambientIntensity;
  vrml_Vect3f attenuation;
  float intensity;
  void Print(ofstream &soubor);
  VRML_POINTLIGHT()
  {
    location=vrml_Vect3f(1000,5000,1000);
    radius=10000;
    vrml_color color=vrml_color(1,1,1);
    ambientIntensity=0.5 ;
    attenuation=vrml_Vect3f(1,0,0);
    intensity=1;
  }

}  ;
//---------------------------------------------------------------------------

class VRML_VIEWPOINT : public vrml_node
{
  public:
  vrml_Rotation transform;
  vrml_Rotation orientation;
  float fieldOfViev;
  vrml_Vect3f position;
  vrml_Vect3f target;
  String description;
  void Print(ofstream &soubor);
  VRML_VIEWPOINT()
  {
    fieldOfViev = M_PI/4;

    position=vrml_Vect3f(0,1000,0);
    target=vrml_Vect3f(200,200,200);
    description="Start";
  }

};
//---------------------------------------------------------------------------
class VRML_FOG : public vrml_node
{
  public:
  vrml_color color;  //  Barva mlhy
  String   fogType ;  //"LINEAR"  Zpsob houstnut mlhy
  float visibilityRange;//  0   Vzdlenost maximlnho dohledu
  void Print(ofstream &soubor);
  VRML_FOG(){color=vrml_color(1,1,1);fogType="LINEAR";visibilityRange=0;}

};

//---------------------------------------------------------------------------
class VRML_ELEVATIONGRID : public vrml_node
{
  public:
  float creaseAngle;
  vrml_Bool solid;
  float xSpacing;
  float zSpacing;
  int xDimension;
  int zDimension;
  float** height;
  int slod,radod;
  //String get_solid(void );//{if(solid) return "TRUE";else return "FALSE";}
  void Print(ofstream &soubor);
  VRML_ELEVATIONGRID()
  {
    creaseAngle=0;solid=true;xSpacing =1; zSpacing=1;
    xDimension=0;
    zDimension=0;
    slod=0;
    radod=0 ;

  }
  VRML_ELEVATIONGRID( float dxSpacing, float dzSpacing, int dxDimension, int dzDimension)
  {
    creaseAngle=0;
    solid=true;
    xSpacing =dxSpacing;
    zSpacing=dzSpacing;
    xDimension=dxDimension;
    zDimension=dzDimension;
    slod=0;
    radod=0 ;

  }
//---------------------------------------------------------------------------
  VRML_ELEVATIONGRID( float dxSpacing, float dzSpacing,int so, int dxDimension,int ro, int dzDimension)
  {
    creaseAngle=0;
    solid=true;
    xSpacing =dxSpacing;
    zSpacing=dzSpacing;
    xDimension=dxDimension;
    zDimension=dzDimension;
    slod=so;
    radod=ro;

  }
} ;


//---------------------------------------------------------------------------

class VRML_MATERIAL : public vrml_node
{
  public:
  double ambientIntensity;
  vrml_color  diffuseColor;
  vrml_color emissiveColor;
  void Print(ofstream &soubor);
  VRML_MATERIAL(){ambientIntensity =0.2;diffuseColor=vrml_color(0.8,0.8,0.8); }
};
//---------------------------------------------------------------------------

class VRML_IMAGETEXTURE : public vrml_node
{
  public:
  String* url;
  int poceturl;
  void Print(ofstream &soubor);
  VRML_IMAGETEXTURE(int pocet){url=new String[pocet];poceturl=pocet;}
  VRML_IMAGETEXTURE(String );//{url=new String[1];poceturl=1;url[0]=a;}
  ~VRML_IMAGETEXTURE()
  {
    delete[] url;
  }
}  ;
//---------------------------------------------------------------------------

class VRML_SHAPE : public vrml_node
{
  public:
  vrml_node* appearance;
  vrml_node* geometry;

  void Print(ofstream &soubor);
  /* VRML_MATERIAL_DATA Material;
   VRML_IMAGETEXTURE_DATA Texture;   */

} ;
//---------------------------------------------------------------------------

class VRML_INDEXEDFACESET : public vrml_node
{
  public:
  int vyska,sirka;
  Hbod3D*  coord;
  int num_coord;
  // vrml_node*  texCoord; //         NULL
  vrml_Bool  ccw;//               TRUE
  troj *mojetroj;
  int num_troj;
  float zkresleni;
  float creaseAngle;//       0         # [0,)
  vrml_Bool  solid;//             TRUE
  // field         MFInt32 texCoordIndex     []        # [-1,)
  void Print(ofstream &soubor);
  VRML_INDEXEDFACESET(){zkresleni=1; ccw=true; creaseAngle=0;solid =true;}

};
//---------------------------------------------------------------------------


class VRML_TRANSFORM : public vrml_node
{
  public :
  vrml_Vect3f    center;  //         0 0 0    # (-,)
  vrml_node*    children[3] ;
  int  mum_children;
  vrml_Rotation rotation ;  //      0 0 1 0  # [-1,1],(-,)
  vrml_Vect3f    scale;          //  1 1 1    # (0,)
  vrml_Rotation scaleOrientation; //0 0 1 0  # [-1,1],(-,)
  vrml_Vect3f    translation;    // 0 0 0    # (-,)
  vrml_Vect3f    bboxCenter;     //  0 0 0    # (-,)
  vrml_Vect3f    bboxSize ;       // -1 -1 -1 # (0,) or -1,-1,-1
  void Print(ofstream &soubor);
  VRML_TRANSFORM()
  {
    center=vrml_Vect3f(0,0,0); mum_children=0;
    rotation=vrml_Rotation(0,0,1,0);
    scaleOrientation=vrml_Rotation(0,0,1,0);
    scale =vrml_Vect3f(1,1,1); translation =vrml_Vect3f(0,0,0);
    bboxCenter=vrml_Vect3f(0,0,0); bboxSize=vrml_Vect3f(-1,-1,-1);
  }


};
//---------------------------------------------------------------------------

class VRML_GROUP : public vrml_node
{
  public:
  vrml_node*    children[3] ;
  int  mum_children;
  vrml_Vect3f    bboxCenter;     //  0 0 0    # (-,)
  vrml_Vect3f    bboxSize ;       // -1 -1 -1 # (0,) or -1,-1,-1

  void Print(ofstream &soubor);
  VRML_GROUP()
  {
    mum_children=0;
    bboxCenter=vrml_Vect3f(0,0,0); bboxSize=vrml_Vect3f(-1,-1,-1);
  }
};

//---------------------------------------------------------------------------
class VRML_APPEARANCE  : public vrml_node
{
  public:
  vrml_node*  material; //         NULL
  vrml_node*  texture;  //         NULL
  vrml_node*  textureTransform; // NULL
  void Print(ofstream &soubor);
  VRML_APPEARANCE () { material=NULL; texture=NULL;textureTransform=NULL;}


};
//---------------------------------------------------------------------------
class VRML_LOD  : public vrml_node
{
  public:
  vrml_node** level;  //
  int num_level;
  vrml_Vect3f center;  // 0 0 0    # (-,)
  float* range ;     // # (0,)
  void Print(ofstream &soubor);
  VRML_LOD (int l) {range=new float [l-1];num_level=0;range[0]=0;level=new vrml_node*[l]; }
};
//---------------------------------------------------------------------------

class VRML_INLINE:public vrml_node
{
  public:

  String url[5];
  int num_url;
  vrml_Vect3f    bboxCenter;     //  0 0 0    # (-,)
  vrml_Vect3f    bboxSize ;       // -1 -1 -1 # (0,) or -1,-1,-1
  void Print(ofstream &soubor);
  VRML_INLINE(){num_url=0; bboxCenter=vrml_Vect3f(0,0,0); bboxSize=vrml_Vect3f(-1,-1,-1);}
  VRML_INLINE(String);//{url[0]=s; num_url=1; bboxCenter=vrml_Vect3f(0,0,0); bboxSize=vrml_Vect3f(-1,-1,-1);}

};

//---------------------------------------------------------------------------
class VRML_TEXTURETRANSFORM:public vrml_node
{
  public:

  vrml_Vect2f center; //  0 0 Poloha vztanho bodu
  float rotation ;  //  0 hel otoen vzhledem ke vztanmu bodu
  vrml_Vect2f scale;  //  1 1 Zmna mtka
  vrml_Vect2f translation;  //  0 0 Posunut
  VRML_TEXTURETRANSFORM ()
  {
    center= vrml_Vect2f(0,0);
    rotation=0;
    scale= vrml_Vect2f(0,0);
    translation= vrml_Vect2f(0,0);
  }
  VRML_TEXTURETRANSFORM (int xs,int ys)
  {
    center= vrml_Vect2f(0,0);
    rotation=0;
    scale= vrml_Vect2f(xs,ys);
    translation= vrml_Vect2f(0,0);
  }
  void Print(ofstream &soubor);
};

#endif

